import { CustomBlockRenderer } from 'react-native-render-html';
import { CustomTagRendererProps } from 'react-native-render-html';
import { DefaultTagRendererProps } from 'react-native-render-html';
import { PropsFromParent } from 'react-native-render-html';
import { default as React_2 } from 'react';
import { TBlock } from 'react-native-render-html';
import { TNode } from 'react-native-render-html';
import { ViewStyle } from 'react-native';

/**
 * @public
 */
export declare interface CellProperties extends Coordinates {
    lenX: number;
    lenY: number;
    constraints: TCellConstraints;
}

/**
 * @public
 */
export declare interface Coordinates {
    x: number;
    y: number;
}

declare interface Display extends Settings {
    maxY: number;
    maxX: number;
    occupiedCoordinates: Array<Coordinates>;
    offsetX: number;
    cells: DisplayCell[];
}

/**
 * @public
 */
export declare interface DisplayCell extends CellProperties {
    tnode: TNode;
}

/**
 * Options to customize this plugin renderers.
 *
 * @public
 */
export declare interface HeuristicTablePluginConfig {
    /**
     * When true, force the table to stretch to the available width.
     */
    forceStretch?: boolean;
    /**
     * Customize cells appearance with this function.
     *
     * @param cell - The cell for which styles should be provided.
     */
    getStyleForCell?(cell: TableCell): ViewStyle | null;
}

/**
 * A component to render tables.
 *
 * @param props - Props from {@link useHtmlTableProps} hook.
 *
 * @public
 */
export declare const HTMLTable: React_2.NamedExoticComponent<HTMLTableProps>;

/**
 * Props for the {@link HTMLTable} component.
 *
 * @public
 */
export declare interface HTMLTableProps extends CustomTagRendererProps<TBlock> {
    layout: TableLayout;
    config: HeuristicTablePluginConfig;
    settings: Settings;
}

/**
 * Renderers to be merged in the `renderers` prop of `RenderHTML` component.
 *
 * @public
 */
declare const renderers: Record<'th' | 'td' | 'table', CustomBlockRenderer>;
export default renderers;

declare interface Settings {
    /**
     * When true, force the table to stretch to the available width.
     */
    forceStretch?: boolean;
    /**
     * Available width prior to scrolling.
     */
    contentWidth: number;
}

/**
 * A cell is a unit of display for one `th` or `td` in a table, with
 * coordinates and lengths relative to the matrix (rows, columns) coordinate
 * system.
 *
 * <pre>
 * 0 1 2 → x
 * 1 . .
 * 2 . .
 * ↓ y
 * </pre>
 *
 * @remarks
 *
 * @public
 */
export declare interface TableCell extends DisplayCell {
    type: 'cell';
    width: number;
}

/**
 * Props received by td and th custom renderers in `propsFromParent` prop
 * field.
 *
 * @public
 */
export declare interface TableCellPropsFromParent extends PropsFromParent {
    config?: HeuristicTablePluginConfig;
    cell: TableCell;
}

/**
 * A container to display items in columns
 * (think 'flex-direction: column')
 *
 * @public
 */
export declare interface TableFlexColumnContainer {
    type: 'col-container';
    children: (TableFlexRowContainer | TableCell)[];
}

/**
 * A container to display items in row.
 * (think 'flex-direction: row')
 *
 * @public
 */
export declare interface TableFlexRowContainer {
    type: 'row-container';
    children: (TableFlexColumnContainer | TableCell)[];
}

declare class TableLayout {
    readonly display: Display;
    readonly columnWidths: number[];
    readonly totalWidth: number;
    readonly renderTree: TableRoot;
    constructor(tnode: TNode, config: Settings);
}

/**
 * A 100% native renderer component for `table` tag.
 *
 * @param props - Component props.
 * @public
 */
export declare const TableRenderer: CustomBlockRenderer;

/**
 * @public
 */
export declare interface TableRoot {
    type: 'root';
    children: TableFlexRowContainer[];
}

/**
 * @public
 */
declare interface TCellConstraints extends TConstraintsBase {
}

/**
 * @public
 */
declare interface TConstraintsBase {
    /**
     * The minimum width for this text.
     */
    minWidth: number;
    /**
     * The total textual density for one column (or cell).
     */
    contentDensity: number;
}

/**
 * The renderer component for `td` tag.
 *
 * @param props - Component props.
 * @public
 */
export declare const TdRenderer: CustomBlockRenderer<TableCellPropsFromParent>;

/**
 * The renderer component for `th` tag.
 *
 * @param props - Component props.
 * @public
 */
export declare const ThRenderer: CustomBlockRenderer<TableCellPropsFromParent>;

/**
 * Customize `td` and `th` renderers while reusing default cell renderer logic.
 *
 * @param props - Props from custom renderer.
 *
 * @public
 */
export declare function useHtmlTableCellProps({ propsFromParent, ...props }: DefaultTagRendererProps<TBlock, TableCellPropsFromParent>): DefaultTagRendererProps<TBlock, TableCellPropsFromParent>;

/**
 * Customize the rendering logic of the table renderer via this hook.
 *
 * @param props - Props from custom renderer props.
 * @param options - Customize this hook behavior.
 *
 * @returns props for the {@link HTMLTable} component.
 *
 * @public
 */
export declare function useHtmlTableProps({ sharedProps, tnode, ...props }: CustomTagRendererProps<TBlock>, options?: {
    /**
     * If present, overrides contentWidth from shared props.
     */
    overrideContentWidth?: number;
}): HTMLTableProps;

export { }
